# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorflow/core/protobuf/device_filters.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n-tensorflow/core/protobuf/device_filters.proto\x12\ntensorflow\"+\n\x11TaskDeviceFilters\x12\x16\n\x0e\x64\x65vice_filters\x18\x01 \x03(\t\"\xa5\x01\n\x10JobDeviceFilters\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x36\n\x05tasks\x18\x02 \x03(\x0b\x32\'.tensorflow.JobDeviceFilters.TasksEntry\x1aK\n\nTasksEntry\x12\x0b\n\x03key\x18\x01 \x01(\x05\x12,\n\x05value\x18\x02 \x01(\x0b\x32\x1d.tensorflow.TaskDeviceFilters:\x02\x38\x01\"B\n\x14\x43lusterDeviceFilters\x12*\n\x04jobs\x18\x01 \x03(\x0b\x32\x1c.tensorflow.JobDeviceFiltersB\x8d\x01\n\x1aorg.tensorflow.distruntimeB\x13\x44\x65viceFiltersProtosP\x01ZUgithub.com/tensorflow/tensorflow/tensorflow/go/core/protobuf/for_core_protos_go_proto\xf8\x01\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tensorflow.core.protobuf.device_filters_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\032org.tensorflow.distruntimeB\023DeviceFiltersProtosP\001ZUgithub.com/tensorflow/tensorflow/tensorflow/go/core/protobuf/for_core_protos_go_proto\370\001\001'
  _JOBDEVICEFILTERS_TASKSENTRY._options = None
  _JOBDEVICEFILTERS_TASKSENTRY._serialized_options = b'8\001'
  _TASKDEVICEFILTERS._serialized_start=61
  _TASKDEVICEFILTERS._serialized_end=104
  _JOBDEVICEFILTERS._serialized_start=107
  _JOBDEVICEFILTERS._serialized_end=272
  _JOBDEVICEFILTERS_TASKSENTRY._serialized_start=197
  _JOBDEVICEFILTERS_TASKSENTRY._serialized_end=272
  _CLUSTERDEVICEFILTERS._serialized_start=274
  _CLUSTERDEVICEFILTERS._serialized_end=340
# @@protoc_insertion_point(module_scope)
