# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorflow/core/framework/function.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tensorflow.core.framework import attr_value_pb2 as tensorflow_dot_core_dot_framework_dot_attr__value__pb2
from tensorflow.core.framework import node_def_pb2 as tensorflow_dot_core_dot_framework_dot_node__def__pb2
from tensorflow.core.framework import op_def_pb2 as tensorflow_dot_core_dot_framework_dot_op__def__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n(tensorflow/core/framework/function.proto\x12\ntensorflow\x1a*tensorflow/core/framework/attr_value.proto\x1a(tensorflow/core/framework/node_def.proto\x1a&tensorflow/core/framework/op_def.proto\"\xa8\x01\n\x12\x46unctionDefLibrary\x12)\n\x08\x66unction\x18\x01 \x03(\x0b\x32\x17.tensorflow.FunctionDef\x12)\n\x08gradient\x18\x02 \x03(\x0b\x32\x17.tensorflow.GradientDef\x12<\n\x14registered_gradients\x18\x03 \x03(\x0b\x32\x1e.tensorflow.RegisteredGradient\"\xc4\x06\n\x0b\x46unctionDef\x12$\n\tsignature\x18\x01 \x01(\x0b\x32\x11.tensorflow.OpDef\x12/\n\x04\x61ttr\x18\x05 \x03(\x0b\x32!.tensorflow.FunctionDef.AttrEntry\x12\x36\n\x08\x61rg_attr\x18\x07 \x03(\x0b\x32$.tensorflow.FunctionDef.ArgAttrEntry\x12P\n\x16resource_arg_unique_id\x18\x08 \x03(\x0b\x32\x30.tensorflow.FunctionDef.ResourceArgUniqueIdEntry\x12%\n\x08node_def\x18\x03 \x03(\x0b\x32\x13.tensorflow.NodeDef\x12-\n\x03ret\x18\x04 \x03(\x0b\x32 .tensorflow.FunctionDef.RetEntry\x12<\n\x0b\x63ontrol_ret\x18\x06 \x03(\x0b\x32\'.tensorflow.FunctionDef.ControlRetEntry\x1a\x42\n\tAttrEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12$\n\x05value\x18\x02 \x01(\x0b\x32\x15.tensorflow.AttrValue:\x02\x38\x01\x1a\x88\x01\n\x08\x41rgAttrs\x12\x38\n\x04\x61ttr\x18\x01 \x03(\x0b\x32*.tensorflow.FunctionDef.ArgAttrs.AttrEntry\x1a\x42\n\tAttrEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12$\n\x05value\x18\x02 \x01(\x0b\x32\x15.tensorflow.AttrValue:\x02\x38\x01\x1aP\n\x0c\x41rgAttrEntry\x12\x0b\n\x03key\x18\x01 \x01(\r\x12/\n\x05value\x18\x02 \x01(\x0b\x32 .tensorflow.FunctionDef.ArgAttrs:\x02\x38\x01\x1a:\n\x18ResourceArgUniqueIdEntry\x12\x0b\n\x03key\x18\x01 \x01(\r\x12\r\n\x05value\x18\x02 \x01(\r:\x02\x38\x01\x1a*\n\x08RetEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x31\n\x0f\x43ontrolRetEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01J\x04\x08\x02\x10\x03\";\n\x0bGradientDef\x12\x15\n\rfunction_name\x18\x01 \x01(\t\x12\x15\n\rgradient_func\x18\x02 \x01(\t\"G\n\x12RegisteredGradient\x12\x15\n\rgradient_func\x18\x01 \x01(\t\x12\x1a\n\x12registered_op_type\x18\x02 \x01(\tB\x80\x01\n\x18org.tensorflow.frameworkB\x0e\x46unctionProtosP\x01ZOgithub.com/tensorflow/tensorflow/tensorflow/go/core/framework/function_go_proto\xf8\x01\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tensorflow.core.framework.function_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\030org.tensorflow.frameworkB\016FunctionProtosP\001ZOgithub.com/tensorflow/tensorflow/tensorflow/go/core/framework/function_go_proto\370\001\001'
  _FUNCTIONDEF_ATTRENTRY._options = None
  _FUNCTIONDEF_ATTRENTRY._serialized_options = b'8\001'
  _FUNCTIONDEF_ARGATTRS_ATTRENTRY._options = None
  _FUNCTIONDEF_ARGATTRS_ATTRENTRY._serialized_options = b'8\001'
  _FUNCTIONDEF_ARGATTRENTRY._options = None
  _FUNCTIONDEF_ARGATTRENTRY._serialized_options = b'8\001'
  _FUNCTIONDEF_RESOURCEARGUNIQUEIDENTRY._options = None
  _FUNCTIONDEF_RESOURCEARGUNIQUEIDENTRY._serialized_options = b'8\001'
  _FUNCTIONDEF_RETENTRY._options = None
  _FUNCTIONDEF_RETENTRY._serialized_options = b'8\001'
  _FUNCTIONDEF_CONTROLRETENTRY._options = None
  _FUNCTIONDEF_CONTROLRETENTRY._serialized_options = b'8\001'
  _FUNCTIONDEFLIBRARY._serialized_start=183
  _FUNCTIONDEFLIBRARY._serialized_end=351
  _FUNCTIONDEF._serialized_start=354
  _FUNCTIONDEF._serialized_end=1190
  _FUNCTIONDEF_ATTRENTRY._serialized_start=742
  _FUNCTIONDEF_ATTRENTRY._serialized_end=808
  _FUNCTIONDEF_ARGATTRS._serialized_start=811
  _FUNCTIONDEF_ARGATTRS._serialized_end=947
  _FUNCTIONDEF_ARGATTRS_ATTRENTRY._serialized_start=742
  _FUNCTIONDEF_ARGATTRS_ATTRENTRY._serialized_end=808
  _FUNCTIONDEF_ARGATTRENTRY._serialized_start=949
  _FUNCTIONDEF_ARGATTRENTRY._serialized_end=1029
  _FUNCTIONDEF_RESOURCEARGUNIQUEIDENTRY._serialized_start=1031
  _FUNCTIONDEF_RESOURCEARGUNIQUEIDENTRY._serialized_end=1089
  _FUNCTIONDEF_RETENTRY._serialized_start=1091
  _FUNCTIONDEF_RETENTRY._serialized_end=1133
  _FUNCTIONDEF_CONTROLRETENTRY._serialized_start=1135
  _FUNCTIONDEF_CONTROLRETENTRY._serialized_end=1184
  _GRADIENTDEF._serialized_start=1192
  _GRADIENTDEF._serialized_end=1251
  _REGISTEREDGRADIENT._serialized_start=1253
  _REGISTEREDGRADIENT._serialized_end=1324
# @@protoc_insertion_point(module_scope)
