# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator/create_python_api.py script.
"""Public API for tf.config.threading namespace.
"""

import sys as _sys

from tensorflow.python.framework.config import get_inter_op_parallelism_threads
from tensorflow.python.framework.config import get_intra_op_parallelism_threads
from tensorflow.python.framework.config import set_inter_op_parallelism_threads
from tensorflow.python.framework.config import set_intra_op_parallelism_threads